@@ -22,6 +22,7 @@
 VERSION=1.0.1
 moduledir="modreqparser modinspector uploadprogress "
 OS=`uname`
+MACHINE=`uname -m`
 ISLINUX=no
 VERSIONNUMBER=
 
@@ -204,20 +205,13 @@ prepareLinux()
             exit 1
         fi
         
-        yum -y install git
-        yum -y install cmake
+        yum -y install git cmake
         installCmake
         
-        yum -y install libtool 
-        yum -y install autoreconf 
-        yum -y install autoheader 
-        yum -y install automake 
-        yum -y install wget 
-        yum -y install go 
-        yum -y install clang 
-        yum -y install patch 
-        yum -y install expat-devel
-        
+        yum -y install libtool autoreconf autoheader automake wget go clang patch expat-devel
+        if [ "${MACHINE}" = "aarch64" ]; then
+            yum -y install libatomic
+        fi
         
     #now for debian and Ubuntu    
     elif [ -f /etc/debian_version ] ; then     
@@ -271,23 +265,16 @@ prepareLinux()
         
         apt-get -y update
         apt-get -f -y install
-        apt-get -y install gcc g++
-        apt-get -y install wget
-        apt-get -y install curl
-        apt-get -y install make
-        
-        apt-get -y install clang 
-        apt-get -y install patch 
-        apt-get -y install libexpat-dev
+        apt-get -y install gcc g++ wget curl make clang patch libexpat-dev
         
         installCmake
-        apt-get -y install git libtool ca-certificates
-        apt-get -y install autotools-dev
-        apt-get -y install autoreconf
-        apt-get -y install autoheader 
-        apt-get -y install automake 
+        apt-get -y install git libtool ca-certificates autotools-dev autoreconf autoheader automake
         installgo
 
+        if [ "${MACHINE}" = "aarch64" ]; then
+            apt-get -y install libatomic1
+        fi
+
     elif [ -f /etc/alpine-release ] ; then
         OSTYPE=ALPINE
         ${APP_MGR_CMD} add make
@@ -406,7 +393,10 @@ updateSrcCMakelistfile()
     commentout 'SET (CMAKE_CXX_COMPILER'   CMakeLists.txt
     
     sed -i -e "s/\${unittest_STAT_SRCS}//g"  src/CMakeLists.txt
-    
+    sed -i -e "s/libstdc++.a //g"  src/CMakeLists.txt
+    sed -i -e "s/-nodefaultlibs //g"  src/CMakeLists.txt
+    sed -i -e "s/-nodefaultlibs libstdc++.a//g"  src/modules/modsecurity-ls/CMakeLists.txt
+
     commentout  ls_llmq.c  src/lsr/CMakeLists.txt
     commentout  ls_llxq.c  src/lsr/CMakeLists.txt
     
@@ -425,6 +415,7 @@ updateSrcCMakelistfile()
     fi
 }
 
+
 updateModuleCMakelistfile()
 {
     echo "cmake_minimum_required(VERSION 2.8)" > src/modules/CMakeLists.txt
@@ -439,12 +430,12 @@ updateModuleCMakelistfile()
         done
     fi
     
-    if [ -f ../thirdparty/lib/libmodsecurity.a ] ; then
+    if [ -f ../third-party/lib/libmodsecurity.a ] ; then
         echo "add_subdirectory(modsecurity-ls)" >> src/modules/CMakeLists.txt
     fi
     
     #For linux but not alpine, add pagespeed module
-    if [ "${ISLINUX}" = "yes" ] ; then
+    if [ "${ISLINUX}" = "yes" ] && [ "${MACHINE}" = "x86_64" ]; then
         if [ ! "${OSTYPE}" = "ALPINE" ] ; then
             echo "add_subdirectory(pagespeed)" >> src/modules/CMakeLists.txt
         fi
@@ -453,6 +444,7 @@ updateModuleCMakelistfile()
     
 }
 
+
 cpModuleSoFiles()
 {
     if [ ! -d dist/modules/ ] ; then
@@ -460,22 +452,19 @@ cpModuleSoFiles()
     fi
     
     for module in ${moduledir}; do
-        cp -f src/modules/${module}/*.so dist/modules/
+        cp -f build/src/modules/${module}/*.so dist/modules/
     done
     
     if [ -e src/modules/modsecurity-ls/mod_security.so ] ; then
-        cp -f src/modules/modsecurity-ls/mod_security.so dist/modules/
+        cp -f build/src/modules/modsecurity-ls/mod_security.so dist/modules/
     fi
     
     if [ -e src/modules/pagespeed/modpagespeed.so ] ; then
-        cp -f src/modules/pagespeed/modpagespeed.so dist/modules/
+        cp -f build/src/modules/pagespeed/modpagespeed.so dist/modules/
     fi
-    
-    
-    
-    
 }
 
+
 fixshmdir()
 {
     if [ ! -d /dev/shm ] ; then
@@ -509,7 +498,7 @@ freebsdFix()
 fixPagespeed()
 {
 PSOLVERSION=1.11.33.4
-cat << EOF > ../thirdparty/psol-$PSOLVERSION/include/pagespeed/kernel/base/scoped_ptr.h
+cat << EOF > ../third-party/psol-$PSOLVERSION/include/pagespeed/kernel/base/scoped_ptr.h
 /**
 * Due the compiling issue, this file was updated from the original file.
 */
@@ -528,9 +517,6 @@ public:
 
 EOF
 
-
-
-
 }
 
 cd `dirname "$0"`
@@ -550,49 +536,57 @@ fi
 
 
 cd ..
-git clone https://github.com/litespeedtech/third-party.git
-mv third-party thirdparty
-mkdir thirdparty/lib64
-cd thirdparty/script/
+if [ ! -d third-party ]; then
 
+    git clone https://github.com/litespeedtech/third-party.git
+    mkdir third-party/lib64
+    cd third-party/script/
 
-#Remove  unittest-cpp and add bcrypt
-sed -i -e "s/unittest-cpp/bcrypt/g" ./build_ols.sh
+    #Remove  unittest-cpp and add bcrypt
+    sed -i -e "s/unittest-cpp/bcrypt/g" ./build_ols.sh
+
+    if [ "${ISLINUX}" != "yes" ] || [ "${MACHINE}" != "x86_64" ] ; then
+        sed -i -e "s/psol/ /g"  ./build_ols.sh
+    fi
+
+    ./build_ols.sh
 
-if [ "${ISLINUX}" != "yes" ] ; then
-    sed -i -e "s/psol/ /g"  ./build_ols.sh
 fi
 
-
-./build_ols.sh
-
 cd ${CURDIR}
 
 updateSrcCMakelistfile
 updateModuleCMakelistfile
 preparelibquic
 
-STDC_LIB=`g++ -print-file-name='libstdc++.a'`
-cp ${STDC_LIB} ../thirdparty/lib64/
-cp ../thirdparty/src/brotli/out/*.a          ../thirdparty/lib64/
-cp ../thirdparty/src//libxml2/.libs/*.a      ../thirdparty/lib64/
-cp ../thirdparty/src/libmaxminddb/include/*  ../thirdparty/include/
 
-if [ "${ISLINUX}" = "yes" ] ; then
+if [ "${ISLINUX}" != "yes" ] || [ "${MACHINE}" != "x86_64" ] ; then
     fixPagespeed
 fi
 
 #special case modsecurity
 cd src/modules/modsecurity-ls
-ln -sf ../../../../thirdparty/src/ModSecurity .
+ln -sf ../../../../third-party/src/ModSecurity .
 cd ../../../
 #Done of modsecurity
 
 fixshmdir
 
-cmake .
-make
-cp src/openlitespeed  dist/bin/
+if [ "x$1" == "xDebug" ]; then
+    BUILD=Debug
+else
+    BUILD=RelWithDebInfo
+fi
+
+if [ ! -d build ]; then
+    mkdir build
+fi
+cd build
+cmake -DCMAKE_BUILD_TYPE=$BUILD ..
+make -j2
+cd ..
+
+cp build/src/openlitespeed  dist/bin/
 
 cpModuleSoFiles
 
