@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -569,7 +569,7 @@ void Awstats::config(HttpVHost *pVHost, int val, char *achBuf, size_t achBufSz,
         if (ConfigCtx::getCurConfigCtx()->getValidPath(achBuf,
                 "$SERVER_ROOT/add-ons/awstats/wwwroot/cgi-bin/",
                 "AWStats CGI-BIN directory"))
-            ; // Ignore result for now
+        {}    ; // Ignore result for now
 
         if (pVHost->getRootContext().determineMime("pl",
                 NULL)->getHandler()->getType())
