@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -41,7 +41,7 @@ public:
     int reinit();
     int run();
     int stop();
-    int linger(int timeout);
+    int linger(int listenerStopped, int timeout);
     void updateDebugLevel();
     //HttpResourceManager * getResManager()
     //{   return &m_ResManager;   }
