@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -381,7 +381,11 @@ int HttpListener::startReusePortSocket(int start, int total)
     int i, ret, fd;
     LS_NOTICE("[%s] Add SO_REUSEPORT socket, #%d to #%d",
                 getAddrStr(), start + 1, total);
-    m_reusePortFds.guarantee(total);
+    m_reusePortFds.guarantee(total - m_reusePortFds.size());
+
+    if (start == 1)
+        ls_setsockopt(m_reusePortFds[0], SOL_SOCKET, SO_REUSEPORT,
+                      (char *)(&start), sizeof(start));
 
     for(i = start; i < total; ++i)
     {
