@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_ev_log.h -- Event logger
  */
@@ -17,6 +17,7 @@ struct lsquic_packet_out;
 struct parse_funcs;
 struct stream_frame;
 struct uncompressed_headers;
+struct stack_st_X509;
 
 
 /* Log a generic event not tied to any particular connection */
@@ -280,8 +281,7 @@ lsquic_ev_log_generated_http_headers (const lsquic_cid_t *, lsquic_stream_id_t,
 void
 lsquic_ev_log_generated_http_push_promise (const lsquic_cid_t *,
         lsquic_stream_id_t stream_id, lsquic_stream_id_t promised_stream_id,
-        const struct lsquic_http_headers *headers,
-        const struct lsquic_http_headers *extra_headers);
+        const struct lsquic_http_headers *headers);
 
 #define EV_LOG_GENERATED_HTTP_PUSH_PROMISE(...) do {                        \
     if (LSQ_LOG_ENABLED_EXT(LSQ_LOG_DEBUG, LSQLM_EVENT))                    \
@@ -311,13 +311,13 @@ lsquic_ev_log_hsk_completed (const lsquic_cid_t *);
 
 
 void
-lsquic_ev_log_zero_rtt (const lsquic_cid_t *);
+lsquic_ev_log_sess_resume (const lsquic_cid_t *);
 
-#define EV_LOG_ZERO_RTT(...) do {                                           \
+#define EV_LOG_SESSION_RESUMPTION(...) do {                                           \
     if (LSQ_LOG_ENABLED_EXT(LSQ_LOG_DEBUG, LSQLM_EVENT))                    \
-        lsquic_ev_log_zero_rtt(__VA_ARGS__);                                \
+        lsquic_ev_log_sess_resume(__VA_ARGS__);                                \
     if (LSQ_LOG_ENABLED_EXT(LSQ_LOG_DEBUG, LSQLM_QLOG))                     \
-        lsquic_qlog_zero_rtt(__VA_ARGS__);                                  \
+        lsquic_qlog_sess_resume(__VA_ARGS__);                                  \
 } while (0)
 
 void
@@ -330,6 +330,16 @@ lsquic_ev_log_check_certs (const lsquic_cid_t *, const lsquic_str_t **, size_t);
         lsquic_qlog_check_certs(__VA_ARGS__);                               \
 } while (0)
 
+void
+lsquic_ev_log_cert_chain (const lsquic_cid_t *, struct stack_st_X509 *);
+
+#define EV_LOG_CERT_CHAIN(...) do {                                         \
+    if (LSQ_LOG_ENABLED_EXT(LSQ_LOG_DEBUG, LSQLM_EVENT))                    \
+        lsquic_ev_log_cert_chain(__VA_ARGS__);                              \
+    if (LSQ_LOG_ENABLED_EXT(LSQ_LOG_DEBUG, LSQLM_QLOG))                     \
+        lsquic_qlog_cert_chain(__VA_ARGS__);                                \
+} while (0)
+
 void
 lsquic_ev_log_version_negotiation (const lsquic_cid_t *, const char *, const char *);
 
