@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * Stream/crypto handshake adapter for the server side.  Since on the server
  * side, the handshake logic is handled in mini conn, this adapter does not
@@ -12,10 +12,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/queue.h>
-#include <sys/socket.h>
-#include <netdb.h>
 #include <time.h>
 
+#ifndef WIN32
+#include <netdb.h>
+#include <sys/socket.h>
+#endif
+
 #include "lsquic_int_types.h"
 #include "lsquic.h"
 
