@@ -1,7 +1,7 @@
 /*
 MIT License
 
-Copyright (c) 2018 - 2020 LiteSpeed Technologies Inc
+Copyright (c) 2018 - 2021 LiteSpeed Technologies Inc
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,11 +31,11 @@ extern "C" {
 
 #include <limits.h>
 #include <stdint.h>
-#include <lsxpack_header.h>
+#include "lsxpack_header.h"
 
 #define LSHPACK_MAJOR_VERSION 2
-#define LSHPACK_MINOR_VERSION 2
-#define LSHPACK_PATCH_VERSION 1
+#define LSHPACK_MINOR_VERSION 3
+#define LSHPACK_PATCH_VERSION 0
 
 #define lshpack_strlen_t lsxpack_strlen_t
 #define LSHPACK_MAX_STRLEN LSXPACK_MAX_STRLEN
@@ -213,6 +213,18 @@ lshpack_dec_set_max_capacity (struct lshpack_dec *, unsigned);
 
 #include <sys/queue.h>
 
+#ifdef __OpenBSD__
+#define STAILQ_HEAD             SIMPLEQ_HEAD
+#define STAILQ_ENTRY            SIMPLEQ_ENTRY
+#define STAILQ_INIT             SIMPLEQ_INIT
+#define STAILQ_INSERT_TAIL      SIMPLEQ_INSERT_TAIL
+#define STAILQ_EMPTY            SIMPLEQ_EMPTY
+#define STAILQ_FIRST            SIMPLEQ_FIRST
+#define STAILQ_NEXT             SIMPLEQ_NEXT
+#define STAILQ_REMOVE_HEAD      SIMPLEQ_REMOVE_HEAD
+#define STAILQ_FOREACH          SIMPLEQ_FOREACH
+#endif
+
 struct lshpack_enc_table_entry;
 
 STAILQ_HEAD(lshpack_enc_head, lshpack_enc_table_entry);
