@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -32,7 +32,7 @@ bool LsiModuleData::initData(int count)
         if (m_iCount > 0)
             return false;
         else
-            delete m_pData;
+            delete [] m_pData;
     }
 
     m_pData = new void_pointer[count];
