@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -132,7 +132,7 @@ public:
     virtual void setLogLevel(const char *pLevel);
     virtual int setAccessLogFile(const char *pFileName, int pipe);
     virtual int setErrorLogFile(const char *pFileName);
-    virtual void setErrorLogRollingSize(off_t size, int keep_days);
+    virtual void setErrorLogRollingSize(off_t size, int keep_days, int compress);
     virtual AccessLog *getAccessLog() const;
 
     void startServing();
