@@ -1,5 +1,20 @@
-// Author: Kevin Fwu
-// Date: August 25, 2015
+/*****************************************************************************
+*    Open LiteSpeed is an open source HTTP server.                           *
+*    Copyright (C) 2013 - 2021  LiteSpeed Technologies, Inc.                 *
+*                                                                            *
+*    This program is free software: you can redistribute it and/or modify    *
+*    it under the terms of the GNU General Public License as published by    *
+*    the Free Software Foundation, either version 3 of the License, or       *
+*    (at your option) any later version.                                     *
+*                                                                            *
+*    This program is distributed in the hope that it will be useful,         *
+*    but WITHOUT ANY WARRANTY; without even the implied warranty of          *
+*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the            *
+*    GNU General Public License for more details.                            *
+*                                                                            *
+*    You should have received a copy of the GNU General Public License       *
+*    along with this program. If not, see http://www.gnu.org/licenses/.      *
+*****************************************************************************/
 
 #ifndef OPENSSL_IS_BORINGSSL
 #include <openssl/err.h>
@@ -245,6 +260,19 @@ static void freeDH(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
 }
 
 
+SSL_CTX *SslUtil::newCtx()
+{
+    SSL_METHOD *meth = (SSL_METHOD *)SSLv23_method();
+    return SSL_CTX_new(meth);
+}
+
+
+void SslUtil::freeCtx(SSL_CTX *pCtx)
+{
+    SSL_CTX_free(pCtx);
+}
+
+
 static long s_iDHParamIdx = -1;
 int SslUtil::initDH(SSL_CTX *pCtx, const char *pFile, int iKeyLen)
 {
@@ -298,7 +326,7 @@ int SslUtil::initECDH(SSL_CTX *pCtx)
     SSL_CTX_set_options(pCtx, SSL_OP_SINGLE_ECDH_USE);
 
 #if (defined SSL_CTRL_SET_ECDH_AUTO || defined SSL_CTX_set_ecdh_auto)
-    SSL_CTX_set_ecdh_auto(pCtx, 1);
+    (void) SSL_CTX_set_ecdh_auto(pCtx, 1);
 #elif (defined SSL_CTX_set1_curves_list || defined SSL_CTRL_SET_CURVES_LIST)
     SSL_CTX_set1_curves_list(pCtx, (char *) "X25519:secp384r1:prime256v1");
 
@@ -372,7 +400,7 @@ int SslUtil::loadPemWithMissingDash(const char *pFile, char *buf, int bufLen,
         pEnd--;
     p = pEnd;
 
-    while (p[-1] == '-')
+    while (p > buf && p[-1] == '-')
         --p;
     while (p + 5 > pEnd)
         *pEnd++ = '-';
@@ -649,6 +677,7 @@ int SslUtil::setCertificateChain(SSL_CTX *pCtx, BIO * bio)
     SSL_CTX_clear_extra_chain_certs(pCtx);
 #endif
     n = 0;
+    ERR_clear_error();
     while ((x509 = PEM_read_bio_X509(bio, NULL, NULL, NULL)) != NULL)
     {
         if (!SSL_CTX_add_extra_chain_cert(pCtx, x509))
